import { system, world, EntityComponentTypes, TicksPerSecond } from "@minecraft/server";
import { RawtextBuilder, showGuidePage, showPage } from "./GuideBook";
import { ModalFormData, ActionFormData } from "@minecraft/server-ui";

export const VERSION = "1.1.0";
export const VERSION_HISTORY = ["1.1.0"];

// Settings for the Desired Paths Addon
const settingsOrder = ["desireSpeed", "desiredPaths"];

const defaultValues = {
  desireSpeed: 25,
  desiredPaths: 1,
};

if (world.scoreboard.getObjective("settings") == undefined) {
  world.scoreboard.addObjective("settings", "settings");
}

settingsOrder.forEach((setting) => {
  const objective = world.scoreboard.getObjective("settings");
  try {
    objective.getScore(setting);
  } catch (error) {
    const defaultValue = defaultValues[setting] || 0;
    objective.setScore(setting, defaultValue);
  }
});
// Configuration for path creation
const pathConfig = {
  pathProgression: [
    { from: "minecraft:grass_block", to: "minecraft:dirt" },
    { from: "minecraft:podzol", to: "minecraft:dirt" },
    { from: "minecraft:dirt", to: "minecraft:coarse_dirt" },
    { from: "minecraft:coarse_dirt", to: "minecraft:grass_path" },
  ],
  baseInterval: 0.25*world.scoreboard.getObjective("settings").getScore("desireSpeed"), // Base interval for checking in ticks (1 second = 20 ticks)
};

const lastModifiedPosition = new Map();

const getRandomBlockToModify = (blocksToCheck) => {
  // Bias towards the center block (index 0) by increasing its chance
  const randomIndex = Math.floor(Math.random() * 7); // Generate random number 0-6
  return randomIndex < 4 ? blocksToCheck[0] : blocksToCheck[randomIndex - 3]; // 4/7 chance for the center, 1/7 chance for each outer block
};

let intervalId;
let currentIntervalTime = pathConfig.baseInterval; // Default interval time

function startInterval(intervalTime) {
  // Clear any existing interval
  if (intervalId) {
    system.clearRun(intervalId);
  }

  // Start a new interval with the specified time
  intervalId = system.runInterval(() => {
    for (const player of world.getAllPlayers()) {
      const playerLocation = player.location;
      const currentPositionKey = `${playerLocation.x},${playerLocation.y},${playerLocation.z}`;

      // Check if the player has moved
      const lastPosition = lastModifiedPosition.get(player);
      if (lastPosition && lastPosition === currentPositionKey) {
        // Player has not moved since the last modification, skip
        continue;
      }

      // Update last position
      lastModifiedPosition.set(player, currentPositionKey);

      const blocksToCheck = [
        { x: playerLocation.x, y: playerLocation.y - 1, z: playerLocation.z },
        { x: playerLocation.x + 1, y: playerLocation.y - 1, z: playerLocation.z },
        { x: playerLocation.x - 1, y: playerLocation.y - 1, z: playerLocation.z },
        { x: playerLocation.x, y: playerLocation.y - 1, z: playerLocation.z + 1 },
        { x: playerLocation.x, y: playerLocation.y - 1, z: playerLocation.z - 1 },
      ];

      const blockPos = getRandomBlockToModify(blocksToCheck);
      const block = player.dimension.getBlock(blockPos);
      if (block) {
        const progression = pathConfig.pathProgression.find(
          (p) => p.from === block.typeId
        );
        if (progression) {
          player.runCommand(
            `setblock ${blockPos.x} ${blockPos.y} ${blockPos.z} ${progression.to}`
          );
        }
      }
    }
  }, intervalTime);
}

function updateInterval(newIntervalTime) {
  currentIntervalTime = newIntervalTime;
  startInterval(currentIntervalTime);
}
startInterval(currentIntervalTime);

const BUTTONS = [
  {
    translation_key: "sg_desired_paths.guide.options.title",
    icon_path: "textures/ui/book_edit_default.png",
    action: (player) => {
      const objective = world.scoreboard.getObjective("settings");

      const form = new ModalFormData()
        .title("Settings")
        .slider(
          `Path Forming Speed`,
          5,
          100,
          5,
          objective.getScore("desireSpeed")
        )
        .toggle(
          "Enable Desired Paths",
          Boolean(objective.getScore("desiredPaths"))
        )
        .submitButton("Save Settings");

      form.show(player).then((res) => {
        if (res.canceled) return;
        const settings = res.formValues;
        settings.forEach((value, index) => {
          const label = settingsOrder[index];
          if (label) {
            objective.setScore(label, value);
          }
        });
        updateInterval(objective.getScore("desireSpeed"))
      });
    },
  },
  {
    // CHANGELOG
    translation_key: "sg_desired_paths.guide.changelog.title",
    icon_path: "textures/ui/book_addtextpage_default.png",
    action: (player) => {
      const body = new RawtextBuilder();
      body.pushLine({
        translate: "sg_desired_paths.guide.changelog.body",
        with: [VERSION],
      });
      body.pushLine();
      for (const version of VERSION_HISTORY) {
        body.pushLine({
          translate: "sg_desired_paths.guide.changelog.subtitle",
          with: [version],
        });
        body.pushLine();
        body.pushLine({
          translate: `sg_desired_paths.guide.changelog.${version}`,
          with: ["\n"],
        });
        body.pushLine();
        body.pushLine();
      }
      showPage(
        player,
        "sg_desired_paths.guide.changelog.title",
        body.build(),
        [
          {
            translation_key: "sg_desired_paths.guide.common.back_button",
            action: (player) => showMainPage(player),
          },
        ]
      );
    },
  },
  {
    translation_key: "sg_desired_paths.guide.common.exit_button",
  },
];

const showMainPage = (player) => {
  showGuidePage(player, "main", 4, BUTTONS)
};

world.afterEvents.itemUse.subscribe((event) => {
  system.run(() => {
    if (event.itemStack.typeId === "sg_desired_paths:guide") {
      showMainPage(event.source);
    }
  });
});
